"use strict";

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _structure = require("../structure");

var _reducers = require("../reducers");

var _forEach = require("lodash/forEach");

var _forEach2 = _interopRequireDefault(_forEach);

var _omit = require("lodash/omit");

var _omit2 = _interopRequireDefault(_omit);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe("reducers", function () {
  describe("calculateDefaults", function () {
    it("Should provide correct default values for a given primitive type", function () {
      expect((0, _reducers.calculateDefaults)(_structure.Types.string("toast"))).toBe("toast");
      expect((0, _reducers.calculateDefaults)(_structure.Types.number(3))).toBe(3);
      expect((0, _reducers.calculateDefaults)(_structure.Types.string())).toBe("");
      expect((0, _reducers.calculateDefaults)(_structure.Types.number())).toBe(0);
    });

    it("Should provide correct default values for an object", function () {
      var objectStructure = _structure.Types.shape({
        test1: _structure.Types.string(),
        test2: _structure.Types.number()
      });
      expect((0, _reducers.calculateDefaults)(objectStructure)).toEqual({
        test1: "",
        test2: 0
      });
    });

    it("Should provide correct default values for nested object", function () {
      var objectStructure = _structure.Types.shape({
        test1: _structure.Types.string(),
        test2: _structure.Types.number(),
        test3: _structure.Types.shape({
          test4: _structure.Types.string("foo")
        })
      });
      expect((0, _reducers.calculateDefaults)(objectStructure)).toEqual({
        test1: "",
        test2: 0,
        test3: {
          test4: "foo"
        }
      });
    });
  });

  describe("determineReducerType", function () {
    it("should return the correct creator function for the default mapping", function () {
      (0, _forEach2.default)(_extends({
        custom: _structure.Types.custom()
      }, (0, _omit2.default)(_structure.Types, "reducer", "wildcardKey", "custom")), function (structureType) {
        var returnVal = (0, _reducers.determineReducerType)(_structure.Types.reducer(structureType()), {
          name: "toast",
          locationString: "toasty"
        });

        expect(_extends({}, returnVal, {
          reducerFn: returnVal.reducerFn.name,
          reducerStructureDescriptor: returnVal.reducerStructureDescriptor.name
        })).toEqual({
          name: "toast",
          reducerFn: _reducers.REDUCER_CREATOR_MAPPING[structureType()().type].name,
          reducerStructureDescriptor: "", //The internal functions should be anonymous
          locationString: "toasty"
        });
      });
    });

    it("should throw an error if the type provided does not match any in the mapping", function () {
      expect(function () {
        return (0, _reducers.determineReducerType)(_structure.Types.reducer(_structure.Types.string()), {
          name: "toast",
          locationString: "toasty",
          reducerCreatorMapping: (0, _omit2.default)(_reducers.REDUCER_CREATOR_MAPPING, _structure.PROP_TYPES._string)
        });
      }).toThrowError(/createReducer/);
    });
  });

  describe("callReducer", function () {
    it("should call the provided reducer with the structure description, location string, and name", function () {
      expect((0, _reducers.callReducer)({
        reducerStructureDescriptor: "foo",
        name: "toast",
        locationString: "toasty",
        reducerFn: function reducerFn(reducerStructureDescriptor, _ref) {
          var locationString = _ref.locationString,
              name = _ref.name;
          return { reducerStructureDescriptor: reducerStructureDescriptor, locationString: locationString, name: name };
        }
      })).toEqual({
        reducerStructureDescriptor: "foo",
        locationString: "toasty",
        name: "toast"
      });
    });
  });

  describe("createReducerBehaviors", function () {
    it("Should return only the reducers of the behavior config and prepend the locationString", function () {
      expect((0, _reducers.createReducerBehaviors)({
        toast: {
          reducer: "foo",
          action: "bar",
          validate: true
        }
      }, "location")).toEqual({
        "location.toast": {
          reducer: "foo",
          action: "bar",
          validate: true
        }
      });
    });
  });
});