"use strict";

var _structure = require("../structure");

function hasType(object, type) {
  return object.type === type;
}

describe("Type descriptions", function () {
  it("will return a function which subsequently returns an object with the correct type", function () {
    expect(hasType(_structure.Types.string()(), _structure.PROP_TYPES._string)).toBeTruthy();
    expect(hasType(_structure.Types.number()(), _structure.PROP_TYPES._number)).toBeTruthy();
    expect(hasType(_structure.Types.boolean()(), _structure.PROP_TYPES._boolean)).toBeTruthy();
    expect(hasType(_structure.Types.arrayOf()(), _structure.PROP_TYPES._array)).toBeTruthy();
    expect(hasType(_structure.Types.reducer()(), _structure.PROP_TYPES._reducer)).toBeTruthy();
    expect(hasType(_structure.Types.shape()(), _structure.PROP_TYPES._shape)).toBeTruthy();
  });

  it("will return the standard default values when none provided", function () {
    expect(_structure.Types.string()().defaultValue).toBe("");
    expect(_structure.Types.number()().defaultValue).toBe(0);
    expect(_structure.Types.boolean()().defaultValue).toBe(false);
    expect(_structure.Types.arrayOf()().defaultValue).toEqual([]);
    expect(_structure.Types.custom()()().defaultValue).toBeUndefined();
  });

  it("will return the default value provided (except for reducer and shape)", function () {
    expect(_structure.Types.string("foo")().defaultValue).toBe("foo");
    expect(_structure.Types.number(5)().defaultValue).toBe(5);
    expect(_structure.Types.boolean(true)().defaultValue).toBe(true);
    expect(_structure.Types.arrayOf(_structure.Types.number(), [1, 2, 3])().defaultValue).toEqual([1, 2, 3]);
    expect(_structure.Types.custom()("foo")().defaultValue).toBe("foo");
  });

  it("will return the correct typeofValue (for string, number, and boolean)", function () {
    expect(_structure.Types.string()().typeofValue).toBe("string");
    expect(_structure.Types.number()().typeofValue).toBe("number");
    expect(_structure.Types.boolean()().typeofValue).toBe("boolean");
  });

  it("will return the correct structure (for arrayOf, reducer, and shape)", function () {
    var structureTest = _structure.Types.string();
    expect(_structure.Types.shape(structureTest)().structure).toEqual(structureTest);
    expect(_structure.Types.arrayOf(structureTest)().structure).toEqual(structureTest);
    expect(_structure.Types.reducer(structureTest)().structure).toEqual(structureTest);
  });

  it("custom value correctly exposes the validator and validation error message properties", function () {
    var customValidator = function customValidator() {
      return false;
    };
    var customErrorMessage = function customErrorMessage() {
      return "Hai!";
    };
    var customType = _structure.Types.custom({
      validator: customValidator,
      validationErrorMessage: customErrorMessage
    })()();

    expect(customType.validator).toEqual(customValidator);
    expect(customType.validationErrorMessage).toEqual(customErrorMessage);
  });

  it("custom value acts the same as any when no configuration provided", function () {
    var unconfiguredCustom = _structure.Types.custom()()();
    expect(unconfiguredCustom.validator("toasty")).toBe(true);
    expect(unconfiguredCustom.validator()).toBe(true);
    expect(unconfiguredCustom.validator(null)).toBe(true);
  });
});