"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createResetAllAction = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.buildStoreChunk = buildStoreChunk;
exports.determineStructureProcessing = determineStructureProcessing;
exports.combineStoreChunkReducers = combineStoreChunkReducers;
exports.processStructure = processStructure;

var _redux = require("redux");

var _find = require("lodash/find");

var _find2 = _interopRequireDefault(_find);

var _omit = require("lodash/omit");

var _omit2 = _interopRequireDefault(_omit);

var _isFunction = require("lodash/isFunction");

var _isFunction2 = _interopRequireDefault(_isFunction);

var _fp = require("lodash/fp");

var _reducers3 = require("./reducers");

var _batchUpdates = require("./reducers/batchUpdates");

var _structure = require("./structure");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

//==============================
// JS imports
//==============================

//==============================
// Flow imports
//==============================


var reduce = require("lodash/fp/reduce").convert({ cap: false });
var map = require("lodash/fp/map").convert({ cap: false });

/* Iterate through our reducers in order to find all reset actions */

/*prettier-ignore*/

/* Iterate through the reducers to find all reset actions, and create a combined reducer
   which will call them all */
var getResetAction = function getResetAction(actions) {
  return (0, _fp.get)("reset")(actions) ? [(0, _fp.get)("reset")(actions)] : reduce(function (resetActions, actions) {
    return [].concat(_toConsumableArray(resetActions), _toConsumableArray(getResetAction(actions)));
  }, [])(actions);
};

var createResetAllAction = exports.createResetAllAction = function createResetAllAction(storeChunkName, chunkActions) {
  return function () {
    return (0, _batchUpdates.createCombinedAction)({
      name: storeChunkName + "-reset-all",
      actions: map(function (action) {
        return action();
      })(getResetAction(chunkActions))
    });
  };
};

// Build a chunk of the eventual store. The selectors and actions
// generated will specifically operate on the store chunk generated. Selectors will be
// relative to the baseSelector provided or, if not specified, the root of the store, using
// the name of the chunk as the base property.
function buildStoreChunk(name, structure) {
  var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref$baseSelector = _ref.baseSelector,
      baseSelector = _ref$baseSelector === undefined ? function (state) {
    return state[name];
  } : _ref$baseSelector,
      _ref$locationString = _ref.locationString,
      locationString = _ref$locationString === undefined ? name : _ref$locationString;

  if (!structure) throw new Error("The structure must be defined for a reducer! LocationString: " + locationString);
  var initialMemo = {
    reducers: _defineProperty({}, name, {}),
    actions: {},
    selectors: {},
    baseSelector: baseSelector,
    locationString: locationString,
    name: name
  };

  //Build up the reducers, actions, and selectors for this level. Due to recursion,
  //these objects will be assigned to a property in the parent object, or simply
  //returned to the call site for use in the rest of the application.
  var processedStructure = determineStructureProcessing(structure, initialMemo, name);

  //If the location string is equal to the name passed to build store chunk, then we must be
  //at the top level. If the structure is a function (i.e. not nested reducers) then return
  //the actions, and selectors as the top level of their respective objects.
  if ((0, _isFunction2.default)(structure)) {
    return {
      reducers: processedStructure.reducers,
      actions: processedStructure.actions[name],
      selectors: processedStructure.selectors[name]
    };
  }

  if (locationString === name) {
    return _extends({}, processedStructure, {
      resetAll: createResetAllAction(name, processedStructure.actions)
    });
  }

  return processedStructure;
}

function determineStructureProcessing(structure, initialMemo, name) {
  if ((0, _isFunction2.default)(structure)) return combineStoreChunkReducers(processStructure(initialMemo, structure, name));
  return combineStoreChunkReducers(reduce(processStructure, initialMemo)(structure));
}

function combineStoreChunkReducers(processedStoreChunk) {
  //The Redux 'combineReducers' helper function is used here to save a little bit of boilerplate.
  //This helper, if you're not aware, ensures that the correct store properties are passed to the
  //reducers assigned to those properties.
  return _extends({}, (0, _omit2.default)(processedStoreChunk, ["baseSelector", "locationString", "name"]), {
    reducers: _defineProperty({}, processedStoreChunk.name, (0, _redux.combineReducers)(processedStoreChunk.reducers))
  });
}

function processStructure(memo, propValue, propName) {
  //Get the structure from the propValue. In the case of 'StructureType' properties, this
  //will be some form of shape (or primitives in the case of arrays). At this point we
  //are only interested in whether or not the structure contains reducers, as that
  //has an impact on how we proceed with regards to calls.
  var _propValue = propValue(),
      propStructure = _propValue.structure;

  var containsReducers = !!(0, _find2.default)(propStructure, function (v) {
    return v().type === _structure.PROP_TYPES._reducer;
  });

  //Create the child reducer. Depending on whether or not the current structure level contains
  //child reducers, we will either recursively call reducerBuilder, or we will call the
  //createReducer function, which will create the correct reducer for the given structure
  //(which can be either object, array, or primitive).
  var childReducer = containsReducers ? buildStoreChunk(propName, propStructure, {
    locationString: memo.locationString + "." + propName,
    baseSelector: function baseSelector(state) {
      return memo.baseSelector(state)[propName];
    }
  }) : (0, _reducers3.createReducer)(propValue, {
    locationString: memo.locationString + "." + propName,
    name: propName
  });

  //As the chunk is built up, we want to assign the reducers/actions created
  //by the child to a location on the reducers/actions object which will match up
  //to their location. Selectors are created at this level, as the child does not
  //need to know where it is located within the grand scheme of things.
  return _extends({}, memo, {
    reducers: _extends({}, memo.reducers, childReducer.reducers),
    actions: _extends({}, memo.actions, _defineProperty({}, propName, childReducer.actions)),
    selectors: _extends({}, memo.selectors, _defineProperty({}, propName, containsReducers ? childReducer.selectors : function (state) {
      return memo.baseSelector(state)[propName];
    }))
  });
}