"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createReducer = exports.createUniqueString = exports.REDUCER_CREATOR_MAPPING = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _REDUCER_CREATOR_MAPP;

exports.determineReducerType = determineReducerType;
exports.callReducer = callReducer;
exports.createReducerBehaviors = createReducerBehaviors;
exports.calculateDefaults = calculateDefaults;

var _structure2 = require("./structure");

var _reduce = require("lodash/reduce");

var _reduce2 = _interopRequireDefault(_reduce);

var _flowRight = require("lodash/fp/flowRight");

var _flowRight2 = _interopRequireDefault(_flowRight);

var _objectReducer = require("./reducers/objectReducer");

var _arrayReducer = require("./reducers/arrayReducer");

var _primitiveReducer = require("./reducers/primitiveReducer");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

//==============================
// JS imports
//==============================

//==============================
// Flow imports
//==============================


//==============================
// Flow types
//==============================


var REDUCER_CREATOR_MAPPING = exports.REDUCER_CREATOR_MAPPING = (_REDUCER_CREATOR_MAPP = {}, _defineProperty(_REDUCER_CREATOR_MAPP, _structure2.PROP_TYPES._shape, _objectReducer.createShapeReducer), _defineProperty(_REDUCER_CREATOR_MAPP, _structure2.PROP_TYPES._array, _arrayReducer.createArrayReducer), _defineProperty(_REDUCER_CREATOR_MAPP, _structure2.PROP_TYPES._boolean, _primitiveReducer.createPrimitiveReducer), _defineProperty(_REDUCER_CREATOR_MAPP, _structure2.PROP_TYPES._string, _primitiveReducer.createPrimitiveReducer), _defineProperty(_REDUCER_CREATOR_MAPP, _structure2.PROP_TYPES._number, _primitiveReducer.createPrimitiveReducer), _defineProperty(_REDUCER_CREATOR_MAPP, _structure2.PROP_TYPES._any, _primitiveReducer.createPrimitiveReducer), _defineProperty(_REDUCER_CREATOR_MAPP, _structure2.PROP_TYPES._custom, _primitiveReducer.createPrimitiveReducer), _REDUCER_CREATOR_MAPP);

var createUniqueString = exports.createUniqueString = Math.random().toString(36).substring(7);

function determineReducerType(reducerDescriptor, _ref) {
  var name = _ref.name,
      locationString = _ref.locationString,
      _ref$reducerCreatorMa = _ref.reducerCreatorMapping,
      reducerCreatorMapping = _ref$reducerCreatorMa === undefined ? REDUCER_CREATOR_MAPPING : _ref$reducerCreatorMa;

  var _reducerDescriptor = reducerDescriptor(),
      structure = _reducerDescriptor.structure;

  var _structure = structure(),
      type = _structure.type;

  if (!reducerCreatorMapping[type]) throw new Error("Reducer type " + type + " does not have a corresponding createReducer function");

  return {
    name: name,
    reducerFn: reducerCreatorMapping[type],
    reducerStructureDescriptor: structure,
    locationString: locationString
  };
}

function callReducer(_ref2) {
  var name = _ref2.name,
      reducerFn = _ref2.reducerFn,
      reducerStructureDescriptor = _ref2.reducerStructureDescriptor,
      locationString = _ref2.locationString;

  return reducerFn(reducerStructureDescriptor, {
    locationString: locationString,
    name: name
  });
}

function createReducerBehaviors(behaviorsConfig, locationString) {
  //Take a reducer behavior config object, and create the reducer behaviors using the location string.
  //This is necessary since all action types are effectively global when Redux processes an action
  //(i.e. every reducer will be ran using the action object). Therefore we need to ensure that all
  //actions only result in the specific reducer performing a change. Actions are also generated using
  //the location string/name combination, so will match up 1:1.
  return (0, _reduce2.default)(behaviorsConfig, function (memo, behavior, name) {
    return _extends({}, memo, _defineProperty({}, locationString + "." + name, behavior));
  }, {});
}

function calculateDefaults(typeDescription) {
  //Using the structure of a type, calculate the default for that type.
  //Types can take two forms; a 'StructureType' and a 'PrimitiveType'. The former
  //can (and usually does) contain nested type descriptions, so we need to recurse
  //through the definition until defaults are found, and build up the corresponding
  //structure.
  var _typeDescription = typeDescription(),
      type = _typeDescription.type,
      _typeDescription$stru = _typeDescription.structure,
      structure = _typeDescription$stru === undefined ? {} : _typeDescription$stru,
      _typeDescription$defa = _typeDescription.defaultValue,
      defaultValue = _typeDescription$defa === undefined ? "" : _typeDescription$defa;

  var complex = [_structure2.PROP_TYPES._array, _structure2.PROP_TYPES._shape].indexOf(type) > -1;

  if (!complex) return defaultValue;

  return (0, _reduce2.default)(structure, function (memo, propValue, propName) {
    return _extends({}, memo, _defineProperty({}, propName, calculateDefaults(propValue)));
  }, {});
}

var createReducer = exports.createReducer = (0, _flowRight2.default)(callReducer, determineReducerType);