'use strict';

var _batchUpdates = require('../batchUpdates');

describe('getApplicableCombinedActions', function () {
  var exampleBehaviors = {
    foo: {
      reducer: function reducer() {}
    }
  };
  var exampleBatchActions = [{ type: 'foo', payload: 'boop' }, { type: 'bar', payload: 'boop' }];

  it('should return an array of all applicable batch actions', function () {
    expect((0, _batchUpdates.getApplicableCombinedActions)(exampleBehaviors)(exampleBatchActions)).toEqual([{ type: 'foo', payload: 'boop' }]);
  });
});


describe('isCombinedAction', function () {
  it('should return true if action contains batch action string', function () {
    expect((0, _batchUpdates.isCombinedAction)('Boop!' + _batchUpdates.COMBINED_ACTION)).toBe(true);
  });

  it('should return false if action does not contain batch action string', function () {
    expect((0, _batchUpdates.isCombinedAction)('')).toBe(false);
  });

  it('should not crash if undefined or null is passed', function () {
    expect((0, _batchUpdates.isCombinedAction)()).toBe(false);
    expect((0, _batchUpdates.isCombinedAction)(null)).toBe(false);
  });
});

describe('createCombinedAction action', function () {
  it('should return an action with a type including the batch update string and name', function () {
    expect((0, _batchUpdates.createCombinedAction)({
      name: 'boop!'
    }).type).toMatch(new RegExp('' + _batchUpdates.COMBINED_ACTION));
  });

  it('should return actions as the payload', function () {
    var example = [];
    expect((0, _batchUpdates.createCombinedAction)({
      actions: example
    }).payload).toBe(example);
  });
});