"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DEFAULT_ARRAY_BEHAVIORS = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createArrayReducer = createArrayReducer;
exports.createReducer = createReducer;
exports.applyValidation = applyValidation;

var _isArray = require("lodash/isArray");

var _isArray2 = _interopRequireDefault(_isArray);

var _isNumber = require("lodash/isNumber");

var _isNumber2 = _interopRequireDefault(_isNumber);

var _validatePayload = require("../validatePayload");

var _reducers2 = require("../reducers");

var _arrayUtils = require("../utils/arrayUtils");

var _structure = require("../structure");

var _batchUpdates = require("./batchUpdates");

var _fp = require("lodash/fp");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

//==============================
// JS imports
//==============================

//==============================
// Flow imports
//==============================


//==============================
// Flow types
//==============================


var reduce = require("lodash/fp/reduce").convert({ cap: false });

function checkIndex(index) {
  var payload = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "";
  var behaviorName = arguments[2];

  if (!(0, _isNumber2.default)(index) || index === -1) {
    console.warn("Index not passed to " + behaviorName + " for payload " + payload + ".");
    return false;
  }
  return true;
}

//==============================
// Array behaviors
// ----------------
// Arrays are more complicated than shape or primitive reducers, due to
// the complexities in amending specific elements. Of course, the behaviours
// could still follow and the same pattern as the other reducers and simply
// make the end user replace the correct index themselves. However, it made sense
// to create a few helper behaviors to aid with the most common array operations.
//==============================
var DEFAULT_ARRAY_BEHAVIORS = exports.DEFAULT_ARRAY_BEHAVIORS = {
  //Index specific behaviors.
  replaceAtIndex: {
    reducer: function reducer(state, payload, initialState, index) {
      if (!checkIndex(index, payload, "updateAtIndex")) return state;
      if (payload === undefined) return console.warn("Undefined was passed when updating index. Update not performed") || state;
      return (0, _arrayUtils.updateAtIndex)(state, payload, index);
    },

    validate: true
  },
  resetAtIndex: {
    reducer: function reducer(state, payload, initialState, index) {
      if (!checkIndex(index, payload, "resetAtIndex")) return state;
      return (0, _arrayUtils.updateAtIndex)(state, initialState, index);
    },

    validate: false
  },
  removeAtIndex: {
    reducer: function reducer(state, index) {
      if (!checkIndex(index, "", "removeAtIndex")) return state;
      return (0, _arrayUtils.removeAtIndex)(state, index);
    },

    validate: false
  },
  //Whole array behaviors.
  replace: {
    reducer: function reducer(state, payload) {
      if (!(0, _isArray2.default)(payload)) return console.warn("An array must be provided when replacing an array") || state;
      return payload;
    },

    validate: true
  },
  reset: {
    reducer: function reducer(state, payload, initialState) {
      return initialState;
    },

    validate: false
  },
  push: {
    reducer: function reducer(state, payload) {
      return [].concat(_toConsumableArray(state), [payload]);
    },

    validate: true
  },
  pushOrRemove: {
    reducer: function reducer(state, payload) {
      var index = (0, _fp.findIndex)((0, _fp.isEqual)(payload))(state);
      if (index == -1) {
        return [].concat(_toConsumableArray(state), [payload]);
      }
      return (0, _arrayUtils.removeAtIndex)(state, index);
    },

    validate: true
  },
  pop: {
    reducer: function reducer(state) {
      return state.slice(0, -1);
    },

    validate: false
  },
  unshift: {
    reducer: function reducer(state, payload) {
      return [payload].concat(_toConsumableArray(state));
    },

    validate: true
  },
  shift: {
    reducer: function reducer(state) {
      return state.slice(1);
    },

    validate: false
  }
};

function createArrayReducer(arrayTypeDescription, _ref) {
  var locationString = _ref.locationString,
      name = _ref.name;

  var uniqueId = Math.random().toString(36).substring(5);
  return {
    reducers: _defineProperty({}, name, createReducer(arrayTypeDescription, (0, _reducers2.createReducerBehaviors)(DEFAULT_ARRAY_BEHAVIORS, uniqueId + "-" + locationString))),
    actions: createActions(DEFAULT_ARRAY_BEHAVIORS, uniqueId + "-" + locationString, {})
  };
}

function createReducer(arrayTypeDescription, behaviors) {
  //Take the initial value specified as the default for the array, then apply it, using the validation
  //when doing so. The initial value must be an array.
  var initialValue = (0, _validatePayload.validateArray)(arrayTypeDescription, arrayTypeDescription().defaultValue);

  //Return the array reducer.
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialValue;
    var _ref2 = arguments[1];
    var type = _ref2.type,
        payload = _ref2.payload,
        index = _ref2.index;

    var matchedBehaviors = behaviors[type] ? [{ type: type, payload: payload }] : (0, _batchUpdates.isCombinedAction)(type) ? (0, _batchUpdates.getApplicableCombinedActions)(behaviors)(payload) : [];

    if (matchedBehaviors.length) {
      //Call every behaviour relevant to this reducer as part of this action call,
      //and merge the result (later actions take priority).
      //Sanitize the payload using the reducer shape, then apply the sanitized
      //payload to the state using the behavior linked to this action type.
      return reduce(function (interimState, matchedBehavior) {
        return behaviors[matchedBehavior.type].reducer(interimState, behaviors[matchedBehavior.type].validate ? applyValidation(arrayTypeDescription, matchedBehavior.payload) : matchedBehavior.payload, initialValue, index);
      }, state)(matchedBehaviors);
    }

    //If the action type does not match any of the specified behaviors, just return the current state.
    return state;
  };
}

function applyValidation(arrayTypeDescription, payload) {
  // Array validation is more tricky than object/primitive, as it is possible that the current
  // action may involve updating the contents of a specific array element, rather than the
  // whole array. As a result, some extra functionality is required to determine which
  // form of validation to apply.

  // First case is simple - if the action payload is an array, then we simply validate it against
  // the structure of this reducer.
  if ((0, _isArray2.default)(payload)) return (0, _validatePayload.validateArray)(arrayTypeDescription, payload);

  // If a non-array payload has been passed in, then we need to check which form of validation
  // to use, by checking the structure of the array.

  var _arrayTypeDescription = arrayTypeDescription(),
      structure = _arrayTypeDescription.structure;

  if (structure().type === _structure.PROP_TYPES._shape) return (0, _validatePayload.validateShape)(structure, payload);
  return (0, _validatePayload.validateValue)(structure, payload);
}

function createActions(behaviorsConfig, locationString) {
  //Take a reducer behavior config object, and create actions using the location string
  return reduce(function (memo, behavior, name) {
    return _extends({}, memo, _defineProperty({}, name, function (payload, index) {
      return {
        type: locationString + "." + name,
        payload: (behavior.action || function (payload) {
          return payload;
        })(payload),
        index: index
      };
    }));
  }, {})(behaviorsConfig);
}