"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DEFAULT_SHAPE_BEHAVIORS = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

//==============================
// JS imports
//==============================


//==============================
// Flow types
//==============================

//==============================
// Flow imports
//==============================


exports.createShapeReducer = createShapeReducer;
exports.calculateDefaults = calculateDefaults;
exports.createReducer = createReducer;

var _isObject = require("lodash/isObject");

var _isObject2 = _interopRequireDefault(_isObject);

var _omit = require("lodash/omit");

var _omit2 = _interopRequireDefault(_omit);

var _validatePayload = require("../validatePayload");

var _reducers2 = require("../reducers");

var _structure = require("../structure");

var _batchUpdates = require("./batchUpdates");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var reduce = require("lodash/fp/reduce").convert({ cap: false });

//==============================
// Shape behaviors
// ----------------
// Shapes differ from primitive reducers by allowing an update behavior, merging the
// payload and the previous state in a shallow way. This supplements the replace
// behavior, which still replaces the previous state with the payload.
//==============================
var DEFAULT_SHAPE_BEHAVIORS = exports.DEFAULT_SHAPE_BEHAVIORS = {
  update: {
    reducer: function reducer(state, payload) {
      if (!(0, _isObject2.default)(payload)) return state;
      return _extends({}, state, payload);
    },

    validate: true
  },
  reset: {
    reducer: function reducer(state, payload, initialState) {
      if (!(0, _isObject2.default)(payload)) return initialState;
      return _extends({}, initialState, payload);
    },

    validate: false
  },
  replace: {
    reducer: function reducer(state, payload, initialState) {
      if (!payload) return state;
      return _extends({}, initialState, payload);
    },

    validate: true
  }
};

function createShapeReducer(reducerShape, _ref) {
  var locationString = _ref.locationString,
      name = _ref.name;

  var uniqueId = Math.random().toString(36).substring(5);
  return {
    reducers: _defineProperty({}, name, createReducer(reducerShape, (0, _reducers2.createReducerBehaviors)(DEFAULT_SHAPE_BEHAVIORS, uniqueId + "-" + locationString))),
    actions: createActions(DEFAULT_SHAPE_BEHAVIORS, uniqueId + "-" + locationString)
  };
}

function calculateDefaults(reducerStructure) {
  return reduce(function (memo, propValue, propName) {
    return _extends({}, memo, _defineProperty({}, propName, propValue().type === _structure.PROP_TYPES._shape ? calculateDefaults(propValue().structure) : propValue().defaultValue));
  }, {})((0, _omit2.default)(reducerStructure, ["_wildcardKey"]));
}

function createReducer(objectStructure, behaviors) {
  var initialState = (0, _validatePayload.validateShape)(objectStructure, calculateDefaults(objectStructure().structure));
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var _ref2 = arguments[1];
    var type = _ref2.type,
        payload = _ref2.payload;

    //If the action type does not match any of the specified behaviors, just return the current state.
    var matchedBehaviors = behaviors[type] ? [{ type: type, payload: payload }] : (0, _batchUpdates.isCombinedAction)(type) ? (0, _batchUpdates.getApplicableCombinedActions)(behaviors)(payload) : [];

    if (matchedBehaviors.length) {
      //Sanitize the payload using the reducer shape, then apply the sanitized
      //payload to the state using the behavior linked to this action type.
      return reduce(function (interimState, matchedBehavior) {
        return behaviors[matchedBehavior.type].reducer(interimState, behaviors[matchedBehavior.type].validate ? (0, _validatePayload.validateShape)(objectStructure, matchedBehavior.payload) : matchedBehavior.payload, initialState);
      }, state)(matchedBehaviors);
    }

    return state;
  };
}

function createActions(behaviorsConfig, locationString) {
  //Take a reducer behavior config object, and create actions using the location string
  return reduce(function (memo, behavior, name) {
    return _extends({}, memo, _defineProperty({}, name, function (payload) {
      return {
        type: locationString + "." + name,
        payload: (behavior.action || function (payload) {
          return payload;
        })(payload)
      };
    }));
  }, {})(behaviorsConfig);
}