"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DEFAULT_PRIMITIVE_BEHAVIORS = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createPrimitiveReducer = createPrimitiveReducer;

var _validatePayload = require("../validatePayload");

var _reducers2 = require("../reducers");

var _batchUpdates = require("./batchUpdates");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

//==============================
// JS imports
//==============================


//==============================
// Flow types
//==============================

//==============================
// Flow imports
//==============================


var reduce = require("lodash/fp/reduce").convert({ cap: false });

//==============================
// Primitive behaviors
// ----------------
// Primitives are the most simple case, as they only have the replace and
// reset behaviors by default.
//==============================

var DEFAULT_PRIMITIVE_BEHAVIORS = exports.DEFAULT_PRIMITIVE_BEHAVIORS = {
  replace: {
    reducer: function reducer(state, payload) {
      if (payload === undefined) return state;
      return payload;
    },

    validate: true
  },
  reset: {
    reducer: function reducer(state, payload, initialState) {
      return initialState;
    },

    validate: false
  }
};

function createPrimitiveReducer(primitiveType, _ref) {
  var locationString = _ref.locationString,
      name = _ref.name;

  var uniqueId = Math.random().toString(36).substring(5);
  return {
    reducers: _defineProperty({}, name, createReducer(primitiveType, (0, _reducers2.createReducerBehaviors)(DEFAULT_PRIMITIVE_BEHAVIORS, uniqueId + "-" + locationString))),
    actions: createActions(DEFAULT_PRIMITIVE_BEHAVIORS, uniqueId + "-" + locationString)
  };
}

function createReducer(primitiveType, behaviors) {
  //Calculate and validate the initial state of the reducer
  var initialState = (0, _validatePayload.validateValue)(primitiveType, primitiveType().defaultValue);
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var _ref2 = arguments[1];
    var type = _ref2.type,
        payload = _ref2.payload;

    //If the action type does not match any of the specified behaviors, just return the current state.
    var matchedBehaviors = behaviors[type] ? [{ type: type, payload: payload }] : (0, _batchUpdates.isCombinedAction)(type) ? (0, _batchUpdates.getApplicableCombinedActions)(behaviors)(payload) : [];

    if (matchedBehaviors.length) {
      //Call every behaviour relevant to this reducer as part of this action call,
      //and merge the result (later actions take priority).
      //Sanitize the payload using the reducer shape, then apply the sanitized
      //payload to the state using the behavior linked to this action type.
      return reduce(function (interimState, matchedBehavior) {
        return behaviors[matchedBehavior.type].reducer(interimState, behaviors[matchedBehavior.type].validate ? (0, _validatePayload.validateValue)(primitiveType, matchedBehavior.payload) : matchedBehavior.payload, initialState);
      }, state)(matchedBehaviors);
    }

    return state;
  };
}

function createActions(behaviorsConfig, locationString) {
  //Take a reducer behavior config object, and create actions using the location string
  return reduce(function (memo, behavior, name) {
    return _extends({}, memo, _defineProperty({}, name, function (payload) {
      return {
        type: locationString + "." + name,
        payload: (behavior.action || function (payload) {
          return payload;
        })(payload)
      };
    }));
  }, {})(behaviorsConfig);
}