"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});


//==============================
// Structure
//==============================
var PROP_TYPES = exports.PROP_TYPES = {
  _string: "_string",
  _number: "_number",
  _boolean: "_boolean",
  _reducer: "_reducer",
  _shape: "_shape",
  _array: "_array",
  _any: "_any",
  _wildcardKey: "_wildcardKey",
  _custom: "_custom"
};

//The types objects are used in order to build up the structure of a store chunk, and provide/accept
//default values whilst doing so.


//==============================
// Flow types
//==============================
var Types = exports.Types = {
  string: function string() {
    var defaultValue = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "";
    return function () {
      return {
        type: PROP_TYPES._string,
        defaultValue: defaultValue,
        typeofValue: "string"
      };
    };
  },
  number: function number() {
    var defaultValue = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    return function () {
      return {
        type: PROP_TYPES._number,
        defaultValue: defaultValue,
        typeofValue: "number"
      };
    };
  },
  boolean: function boolean() {
    var defaultValue = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
    return function () {
      return {
        type: PROP_TYPES._boolean,
        defaultValue: defaultValue,
        typeofValue: "boolean"
      };
    };
  },
  any: function any() {
    var defaultValue = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
    return function () {
      return {
        type: PROP_TYPES._any,
        defaultValue: defaultValue,
        typeofValue: "any"
      };
    };
  },
  arrayOf: function arrayOf(structure) {
    var defaultValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    return function () {
      return {
        type: PROP_TYPES._array,
        structure: structure,
        defaultValue: defaultValue
      };
    };
  },
  reducer: function reducer(structure) {
    return function () {
      return {
        type: PROP_TYPES._reducer,
        structure: structure
      };
    };
  },
  shape: function shape(structure) {
    return function () {
      return {
        type: PROP_TYPES._shape,
        structure: structure
      };
    };
  },
  custom: function custom() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        _ref$validator = _ref.validator,
        validator = _ref$validator === undefined ? function () {
      return true;
    } : _ref$validator,
        _ref$validationErrorM = _ref.validationErrorMessage,
        validationErrorMessage = _ref$validationErrorM === undefined ? function (value) {
      return value + " failed custom type validation";
    } : _ref$validationErrorM;

    return function (defaultValue) {
      return function () {
        return {
          type: PROP_TYPES._custom,
          defaultValue: defaultValue,
          validator: validator,
          validationErrorMessage: validationErrorMessage
        };
      };
    };
  },
  wildcardKey: function wildcardKey() {
    return PROP_TYPES._wildcardKey;
  }
};