'use strict';

var _arrayUtils = require('../arrayUtils');

describe('Array utilities', function () {
    describe('updateAtIndex', function () {
        it('Should throw error if attempting to update without an index', function () {
            expect(function () {
                return (0, _arrayUtils.updateAtIndex)([1, 2, 3], 4);
            }).toThrowError(/index/);
        });

        it('Should throw error if attempting to update with a non numeric index', function () {
            expect(function () {
                return (0, _arrayUtils.updateAtIndex)([1, 2, 3], 4, 'toast');
            }).toThrowError(/index/);
        });

        it('Should throw error if attempting to update with an out of range index', function () {
            expect(function () {
                return (0, _arrayUtils.updateAtIndex)([1, 2, 3], 4, 4);
            }).toThrowError(/index/);
        });

        it('Should update value at the correct index', function () {
            expect((0, _arrayUtils.updateAtIndex)([1, 2, 3], 4, 0)).toEqual([4, 2, 3]);
            expect((0, _arrayUtils.updateAtIndex)([1, 2, 3], 4, 1)).toEqual([1, 4, 3]);
            expect((0, _arrayUtils.updateAtIndex)([1, 2, 3], 4, 2)).toEqual([1, 2, 4]);
        });
    });

    describe('removeAtIndex', function () {
        it('Should throw error if attempting to remove without an index', function () {
            expect(function () {
                return (0, _arrayUtils.removeAtIndex)([1, 2, 3]);
            }).toThrowError(/index/);
        });

        it('Should throw error if attempting to remove with a non numeric index', function () {
            expect(function () {
                return (0, _arrayUtils.removeAtIndex)([1, 2, 3], 'toast');
            }).toThrowError(/index/);
        });

        it('Should throw error if attempting to remove with an out of range index', function () {
            expect(function () {
                return (0, _arrayUtils.removeAtIndex)([1, 2, 3], 4);
            }).toThrowError(/index/);
        });

        it('Should remove value at the correct index', function () {
            expect((0, _arrayUtils.removeAtIndex)([1, 2, 3], 0)).toEqual([2, 3]);
            expect((0, _arrayUtils.removeAtIndex)([1, 2, 3], 1)).toEqual([1, 3]);
            expect((0, _arrayUtils.removeAtIndex)([1, 2, 3], 2)).toEqual([1, 2]);
        });
    });
});