"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getValueType = exports.hasWildcardKey = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

//==============================
// JS imports
//==============================

//==============================
// Flow imports and types
//==============================


exports.validateShape = validateShape;
exports.validateValue = validateValue;
exports.validateArray = validateArray;
exports.getTypeValidation = getTypeValidation;

var _reduce = require("lodash/reduce");

var _reduce2 = _interopRequireDefault(_reduce);

var _isObject = require("lodash/isObject");

var _isObject2 = _interopRequireDefault(_isObject);

var _structure = require("./structure");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var find = require("lodash/fp/find").convert({ cap: false });

var hasWildcardKey = exports.hasWildcardKey = function hasWildcardKey(objectStructure) {
  return !!find(function (prop, key) {
    return key === _structure.PROP_TYPES._wildcardKey;
  })(objectStructure().structure);
};

var getValueType = exports.getValueType = function getValueType(objectStructure, key, wildcardKeyPresent) {
  return wildcardKeyPresent ? objectStructure().structure[key] || objectStructure().structure[_structure.PROP_TYPES._wildcardKey] : objectStructure().structure[key];
};

function validateShape(objectStructure, value) {
  if (!(0, _isObject2.default)(value)) {
    console.error("The value passed to validateObject() was not an object. Value: ", value);
    return {};
  }

  var wildcardKeyPresent = hasWildcardKey(objectStructure);

  return (0, _reduce2.default)(value, function (memo, value, name) {
    var valueType = getValueType(objectStructure, name, wildcardKeyPresent);
    //If the value type does not exist in the reducer structure, and there's no wildcard key, then
    //we don't want to include it in the payload.
    //Display a console error for the developer, and skip the inclusion of this property in the payload.
    if (!valueType) {
      console.warn("The property, " + name + ", was not specified in the structure" + (" and was stripped out of the payload. Structure: " + objectStructure().structure));
      return memo;
    }

    var validatedValue = getTypeValidation(valueType().type)(valueType, value);

    if (validatedValue === undefined) {
      console.warn("The property, " + name + ", was populated with a type " + (typeof value === "undefined" ? "undefined" : _typeof(value)) + " which does not" + (" match that specified in the reducer configuration " + (wildcardKeyPresent ? ", nor did it match a wildcardKey" : "") + ". It has been stripped from") + " the payload");
      return memo;
    }

    return _extends({}, memo, _defineProperty({}, name, validatedValue));
  }, {});
}

function validateValue(primitive, value) {
  var evaluatedPrimitive = primitive();
  //If this value is a custom value, then we should apply it's custom validator!
  if (evaluatedPrimitive.type === _structure.PROP_TYPES._custom) {
    if (evaluatedPrimitive.validator(value)) return value;
    return console.warn(evaluatedPrimitive.validationErrorMessage(value));
  }

  //Otherwise we will use the standard, basic, typeof checks.
  if ((typeof value === "undefined" ? "undefined" : _typeof(value)) === evaluatedPrimitive.typeofValue || evaluatedPrimitive.typeofValue === "any") return value;
  return console.warn("The value, " + value + ", did not match the type specified (" + evaluatedPrimitive.type + ").");
}

function validateArray(arrayStructure, value) {
  //Validate arrays by performing either of the other validation types to each element of the array,
  //based on the provided reducer structure.
  if (!Array.isArray(value)) {
    console.error("The value passed to validateArray() was not an array. Value: ", value);
    return [];
  }
  var elementStructure = arrayStructure().structure;
  var elementType = elementStructure().type;
  return value.map(function (element) {
    return getTypeValidation(elementType)(elementStructure, element);
  }).filter(function (e) {
    return e;
  });
}

function getTypeValidation(type) {
  var _TYPE_VALIDATIONS;

  var TYPE_VALIDATIONS = (_TYPE_VALIDATIONS = {}, _defineProperty(_TYPE_VALIDATIONS, _structure.PROP_TYPES._string, validateValue), _defineProperty(_TYPE_VALIDATIONS, _structure.PROP_TYPES._number, validateValue), _defineProperty(_TYPE_VALIDATIONS, _structure.PROP_TYPES._boolean, validateValue), _defineProperty(_TYPE_VALIDATIONS, _structure.PROP_TYPES._array, validateArray), _defineProperty(_TYPE_VALIDATIONS, _structure.PROP_TYPES._shape, validateShape), _defineProperty(_TYPE_VALIDATIONS, _structure.PROP_TYPES._any, validateValue), _defineProperty(_TYPE_VALIDATIONS, _structure.PROP_TYPES._custom, validateValue), _TYPE_VALIDATIONS);
  var typeValidation = TYPE_VALIDATIONS[type];
  if (!typeValidation) {
    throw new Error("The type " + type + " does not have a corresponding\n                validation function!");
  }
  return typeValidation;
}